@@ -6,7 +6,7 @@ use warnings;
 use Algorithm::Combinatorics qw(subsets variations);
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.13;
+our $VERSION = 0.14;
 
 my$self;
 
@@ -33,8 +33,8 @@ sub new{
 		},
 		output => undef,
 		messages => undef,
-		itemsKeyDescription => undef,
-		itemsKeyTransactions => undef,
+		keyItemsDescription => undef,
+		keyItemsTransactions => undef,
 		limitRules => undef,
 		frequentItemset => [],
 		associationRules => undef,
@@ -47,28 +47,32 @@ sub new{
 }
 
 sub validate_data{
-	(defined $self->{itemsKeyDescription})
-		or die("Error: \$apriori->{itemsKeyDescription} is not defined!\n");
-	(defined $self->{itemsKeyTransactions})
-		or die("Error: \$apriori->{itemsKeyTransactions} is not defined!\n");
+	(defined $self->{keyItemsDescription})
+		or die('Error: $apriori->{keyItemsDescription} is not defined!');
+	(defined $self->{keyItemsTransactions})
+		or die('Error: $apriori->{keyItemsTransactions} is not defined!');
+	(defined $self->{metrics}{minSupport})
+		or die('Error: $apriori->{metrics}{minSupport} is not defined!');
+	(defined $self->{metrics}{minConfidence})
+		or die('Error: $apriori->{metrics}{minConfidence} is not defined!');
 }
 
 sub insert_key_items_transaction{
 	(scalar(@_)==2)
-		or die("Error: \$apriori->insert_key_items_transaction(\\\@items) missing parameter key items to array reference!\n");
+		or die('Error: $apriori->insert_key_items_transaction(\@items) missing parameter key items to array reference!');
 	(ref($_[1]) eq "ARRAY")
-		or die("Error: \$apriori->insert_key_items_transaction(\\\@items) parameter key items is not an array reference!\n");
+		or die('Error: $apriori->insert_key_items_transaction(\@items) parameter key items is not an array reference!');
 	my@items=sort(@{$_[1]});
 	my @itemsets=subsets(\@items);
 	foreach my$itemset(@itemsets){
-		$self->{itemsKeyTransactions}{"@{$itemset}"}++;
+		$self->{keyItemsTransactions}{"@{$itemset}"}++;
 	}
 	$self->{totalTransactions}++;
 }
 
 sub input_data_file{
 	(scalar(@_)==3)
-		or die("Error: \$apriori->input_data_file(\"datafile.txt\",\",\") missing parameters path to data file and/or item separator!\n");
+		or die('Error: $apriori->input_data_file("datafile.txt",",") missing parameters path to data file and/or item separator!');
 	my$file=$_[1];
 	my$separator=$_[2];
 	open(FILE,"<$file")
@@ -85,17 +89,16 @@ sub input_data_file{
 
 sub quantity_possible_rules{
 	$self->validate_data;
-	return ((3**scalar(keys(%{$self->{itemsKeyDescription}})))-(2**(scalar(keys(%{$self->{itemsKeyDescription}}))+1))+1);
+	return ((3**scalar(keys(%{$self->{keyItemsDescription}})))-(2**(scalar(keys(%{$self->{keyItemsDescription}}))+1))+1);
 }
 
 sub generate_rules{
 	$self->validate_data;
-	my @largeItemsetSizeOne = keys(%{$self->{itemsKeyDescription}});
 	if($self->{messages}){
-		print "\n${\scalar(keys(%{$self->{itemsKeyDescription}}))} items, ${\$self->quantity_possible_rules} possible rules";
+		print "\n${\scalar(keys(%{$self->{keyItemsDescription}}))} items, ${\$self->quantity_possible_rules} possible rules";
 	}
-	my @frequentItemset = grep{(($self->{itemsKeyTransactions}{$_}/$self->{totalTransactions})*100)>=$self->{metrics}{minSupport}}@largeItemsetSizeOne;
-	$self->association_rules(\@frequentItemset);
+	my @largeItemsetSizeOne = grep{(($self->{keyItemsTransactions}{$_}/$self->{totalTransactions})*100)>=$self->{metrics}{minSupport}}keys(%{$self->{keyItemsDescription}});
+	$self->association_rules(\@largeItemsetSizeOne);
 }
 
 sub association_rules{
@@ -118,15 +121,15 @@ sub association_rules{
 			}
 			@antecedent=sort@antecedent;
 			@consequent=sort@consequent;
-			next if("@consequent" ~~ @{$self->{implications}{"@antecedent"}});
+			next if("@consequent"~~@{$self->{implications}{"@antecedent"}});
 			push @{$self->{implications}{"@antecedent"}},"@consequent";
 			my@implication;
 			push@implication,@antecedent,@consequent;
 			@implication=sort(@implication);
-			next unless $self->{itemsKeyTransactions}{"@antecedent"};
-			my$supportAntecedent=($self->{itemsKeyTransactions}{"@antecedent"}/$self->{totalTransactions});
-			next unless $self->{itemsKeyTransactions}{"@implication"};
-			my$supportConsequent=($self->{itemsKeyTransactions}{"@implication"}/$self->{totalTransactions});
+			next unless $self->{keyItemsTransactions}{"@antecedent"};
+			my$supportAntecedent=($self->{keyItemsTransactions}{"@antecedent"}/$self->{totalTransactions});
+			next unless $self->{keyItemsTransactions}{"@implication"};
+			my$supportConsequent=($self->{keyItemsTransactions}{"@implication"}/$self->{totalTransactions});
 			my $support = $supportConsequent;
 			next if($support < $self->{metrics}{minSupport});
 			my $confidence = ($supportConsequent/$supportAntecedent);
@@ -148,18 +151,28 @@ sub association_rules{
 			my $jaccard = ($support/($supportAntecedent+$supportConsequent-$support));
 			next if(defined $self->{metrics}{minJaccard} && $jaccard < $self->{metrics}{minJaccard});
 			$self->{rule}++;
+			$support = sprintf("%.4f", $support);
+			$confidence = sprintf("%.4f", $confidence);
+			$lift = sprintf("%.4f", $lift);
+			$leverage = sprintf("%.4f", $leverage);
+			$conviction = sprintf("%.4f", $conviction)if($conviction ne "NaN");
+			$coverage = sprintf("%.4f", $coverage);
+			$correlation = sprintf("%.4f", $correlation);
+			$cosine = sprintf("%.4f", $cosine);
+			$laplace = sprintf("%.4f", $laplace);
+			$jaccard = sprintf("%.4f", $jaccard);
 			$self->{associationRules}{$self->{rule}} = {
 				implication => "{ @antecedent } => { @consequent }",
-				support => sprintf("%.4f", $support), 
-				confidence => sprintf("%.4f", $confidence),
-				lift => sprintf("%.4f", $lift),
-				leverage => sprintf("%.4f", $leverage),
-				conviction => sprintf("%.4f", $conviction),
-				coverage => sprintf("%.4f", $coverage),
-				correlation => sprintf("%.4f", $correlation),
-				cosine => sprintf("%.4f", $cosine),
-				laplace => sprintf("%.4f", $laplace),
-				jaccard => sprintf("%.4f", $jaccard),
+				support => $support, 
+				confidence => $confidence,
+				lift => $lift,
+				leverage => $leverage,
+				conviction => $conviction,
+				coverage => $coverage,
+				correlation => $correlation,
+				cosine => $cosine,
+				laplace => $laplace,
+				jaccard => $jaccard,
 				items => [@antecedent, @consequent]
 			};
 			my@items=grep{!($_~~@frequentItemset)}@implication;
@@ -230,6 +243,8 @@ sub file{
 	}
 	print FILE "\n";
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
+		$self->{associationRules}{$rule}{support}=~s/\./,/;
+		$self->{associationRules}{$rule}{confidence}=~s/\./,/;
 		foreach my$metric(@metrics){
 			$self->{associationRules}{$rule}{lc$metric}=~s/\./,/;
 		}
@@ -249,14 +264,14 @@ sub file{
 		}
 		print FILE "Items:\n";
 		foreach my$item(@{$self->{associationRules}{$rule}{items}}){
-			print FILE "$item $self->{itemsKeyDescription}{$item}\n";
+			print FILE "$item $self->{keyItemsDescription}{$item}\n";
 		}
 		print FILE "\n";
 	}
 	print FILE "Frequent itemset: { @{$self->{frequentItemset}} }\n";
 	print FILE "Items:\n";
 	foreach my$item(@{$self->{frequentItemset}}){
-		print FILE "$item $self->{itemsKeyDescription}{$item}\n";
+		print FILE "$item $self->{keyItemsDescription}{$item}\n";
 	}
 	close(FILE);
 }
@@ -302,7 +317,7 @@ sub excel{
 	push @data,\@jaccard if defined $self->{metrics}{minJaccard};
 	$worksheet->write('A1', $headings, $bold);
 	$worksheet->write('A2', \@data);
-	my$chart=$workbook->add_chart(type => 'column', embedded => 1);
+	my$chart=$workbook->add_chart(type =>'column', embedded=>1);
 	my@columns=('B'..'M');
 	my$i=0;
 	$chart->add_series(
@@ -340,7 +355,7 @@ sub excel{
 		$worksheet->write("A$line","Items:");
 		$line++;
 		foreach my$item(@{$self->{associationRules}{$rule}{items}}){
-			$worksheet->write("A$line","$item $self->{itemsKeyDescription}{$item}");
+			$worksheet->write("A$line","$item $self->{keyItemsDescription}{$item}");
 			$line++;
 		}
 		$line++;
@@ -350,7 +365,7 @@ sub excel{
 	$worksheet->write("A$line","Items:");
 	$line++;
 	foreach my$item(@{$self->{frequentItemset}}){
-		$worksheet->write("A$line","$item $self->{itemsKeyDescription}{$item}");
+		$worksheet->write("A$line","$item $self->{keyItemsDescription}{$item}");
 		$line++;
 	}
 	$workbook->close;
@@ -382,9 +397,9 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	my $apriori = new Data::Mining::Apriori;
 
-	$apriori->{metrics}{minSupport}=0.0155; # The minimum support, default value is 0.01(1%)
+	$apriori->{metrics}{minSupport}=0.0155; # The minimum support(required), default value is 0.01(1%)
 
-	$apriori->{metrics}{minConfidence}=0.0155; # The minimum confidence, default value is 0.10(10%)
+	$apriori->{metrics}{minConfidence}=0.0155; # The minimum confidence(required), default value is 0.10(10%)
 
 	$apriori->{metrics}{minLift}=1; # The minimum lift(optional)
 
@@ -403,16 +418,16 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	$apriori->{metrics}{minJaccard}=0; # The minimum jaccard(optional)
 
 	$apriori->{output}=1;
-	# The output type (1 - Export to text file delimited by tab; 2 - Export to excel file with chart)(optional)
+	# The output type(optional): 1 - Export to text file delimited by tab; 2 - Export to excel file with chart.
 
 	$apriori->{messages}=1; # A value boolean to display the messages(optional)
 
-	$apriori->{itemsKeyDescription}{'101'}='MILK'; # Hash table to add items by key and description
-	$apriori->{itemsKeyDescription}{102}='BREAD';
-	$apriori->{itemsKeyDescription}{'103'}='CEREAL';
+	$apriori->{keyItemsDescription}{'101'}='MILK'; # Hash table reference to add items by key and description
+	$apriori->{keyItemsDescription}{102}='BREAD';
+	$apriori->{keyItemsDescription}{'103'}='CEREAL';
 
 	my@items=(103,101);
-	$apriori->insert_key_items_transaction(\@items); # Insert key items per transaction
+	$apriori->insert_key_items_transaction(\@items); # Insert key items by transaction
 	$apriori->insert_key_items_transaction([103,102]);
 	$apriori->insert_key_items_transaction([103,101,102]);
 	$apriori->insert_key_items_transaction([103,101,102]);
@@ -426,7 +441,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	# or from a data file
 
 	$apriori->input_data_file("datafile.txt",",");
-	# Insert items per line(transaction), accepts the arguments of path to data file and item separator
+	# Insert key items by line(transaction), accepts the arguments of path to data file and item separator
 
 	# file contents (example)
 
@@ -446,7 +461,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	$apriori->{limitRules}=10; # The limit of rules
 
 	$apriori->generate_rules;
-	# Generate association rules to no longer meet the minimum support, confidence, lift, leverage, conviction, coverage or limit of rules
+	# Generate association rules to no longer meet the minimum support, confidence, lift, leverage, conviction, coverage, correlation, cosine, laplace, jaccard or limit of rules
 
 	print "\n@{$apriori->{frequentItemset}}\n"; # Show frequent items
 
@@ -467,43 +482,41 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	#output file "output_itemset_size_2.txt"
 
 	Rules	Support	Confidence	Lift	Leverage	Conviction	Coverage	Correlation	Cosine	Laplace	Jaccard
-	R1	0.7000	0.7778	1,1111	0,0700	1,3500	0,9000	0,5092	0,8819	0,5862	0,7778
-	R2	0.8000	0.8889	1,1111	0,0800	1,8000	0,9000	0,6667	0,9428	0,6207	0,8889
-	R3	0.7000	0.8750	1,2500	0,1400	2,4000	0,8000	0,7638	0,9354	0,6071	0,8750
-	R4	0.7000	0.8750	1,2500	0,1400	2,4000	0,8000	0,7638	0,9354	0,6071	0,8750
-	R5	0.8000	0.8889	1,1111	0,0800	1,8000	0,9000	0,6667	0,9428	0,6207	0,8889
-	R6	0.7000	0.7778	1,1111	0,0700	1,3500	0,9000	0,5092	0,8819	0,5862	0,7778
+	R1	0,7000	0,8750	1,2500	0,1400	2,4000	0,8000	0,7638	0,9354	0,6071	0,8750
+	R2	0,7000	0,8750	1,2500	0,1400	2,4000	0,8000	0,7638	0,9354	0,6071	0,8750
+	R3	0,7000	0,7778	1,1111	0,0700	1,3500	0,9000	0,5092	0,8819	0,5862	0,7778
+	R4	0,8000	0,8889	1,1111	0,0800	1,8000	0,9000	0,6667	0,9428	0,6207	0,8889
+	R5	0,7000	0,7778	1,1111	0,0700	1,3500	0,9000	0,5092	0,8819	0,5862	0,7778
+	R6	0,8000	0,8889	1,1111	0,0800	1,8000	0,9000	0,6667	0,9428	0,6207	0,8889
 
-	Rule R1: { 102 } => { 101 }
-	Support: 0.7000
-	Confidence: 0.7778
-	Lift: 1,1111
-	Leverage: 0,0700
-	Conviction: 1,3500
-	Coverage: 0,9000
-	Correlation: 0,5092
-	Cosine: 0,8819
-	Laplace: 0,5862
-	Jaccard: 0,7778
+	Rule R1: { 101 } => { 103 }
+	Support: 0,7000
+	Confidence: 0,8750
+	Lift: 1,2500
+	Leverage: 0,1400
+	Conviction: 2,4000
+	Coverage: 0,8000
+	Correlation: 0,7638
+	Cosine: 0,9354
+	Laplace: 0,6071
+	Jaccard: 0,8750
 	Items:
-	102 BREAD
 	101 MILK
+	103 CEREAL
 
 	to be continued...
 
 	#output file "output_itemset_size_3.txt"
 
 	Rules	Support	Confidence	Lift	Leverage	Conviction	Coverage	Correlation	Cosine	Laplace	Jaccard
-	R7	0.6000	0.7500	1,2500	0,1200	1,6000	0,8000	0,6124	0,8660	0,5714	0,7500
-	R8	0.6000	0.8571	1,4286	0,1800	2,8000	0,7000	0,8018	0,9258	0,5926	0,8571
-	R9	0.6000	0.8571	1,4286	0,1800	2,8000	0,7000	0,8018	0,9258	0,5926	0,8571
-	R10	0.6000	0.6667	1,1111	0,0600	1,2000	0,9000	0,4082	0,8165	0,5517	0,6667
-	R11	0.6000	0.7500	1,2500	0,1200	1,6000	0,8000	0,6124	0,8660	0,5714	0,7500
-	R12	0.6000	0.6667	1,1111	0,0600	1,2000	0,9000	0,4082	0,8165	0,5517	0,6667
+	R7	0,6000	0,7500	1,2500	0,1200	1,6000	0,8000	0,6124	0,8660	0,5714	0,7500
+	R8	0,6000	0,8571	1,4286	0,1800	2,8000	0,7000	0,8018	0,9258	0,5926	0,8571
+	R9	0,6000	0,8571	1,4286	0,1800	2,8000	0,7000	0,8018	0,9258	0,5926	0,8571
+	R10	0,6000	0,6667	1,1111	0,0600	1,2000	0,9000	0,4082	0,8165	0,5517	0,6667
 
 	Rule R7: { 101 } => { 102, 103 }
-	Support: 0.6000
-	Confidence: 0.7500
+	Support: 0,6000
+	Confidence: 0,7500
 	Lift: 1,2500
 	Leverage: 0,1200
 	Conviction: 1,6000
@@ -517,9 +530,9 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	102 BREAD
 	103 CEREAL
 
-	Rule R8: { 101, 102 } => { 103 }
-	Support: 0.6000
-	Confidence: 0.8571
+	Rule R8: { 101, 103 } => { 102 }
+	Support: 0,6000
+	Confidence: 0,8571
 	Lift: 1,4286
 	Leverage: 0,1800
 	Conviction: 2,8000
@@ -530,8 +543,8 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	Jaccard: 0,8571
 	Items:
 	101 MILK
-	102 BREAD
 	103 CEREAL
+	102 BREAD
 
 	to be continued...
 
@@ -633,18 +646,18 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	foreach my$transaction(@$transactions){
 		$sql = qq~
-			SELECT dp.product_alternate_key, dp.product_name
-			FROM dimension_product dp
-			JOIN fact_sales fs ON
-			dp.product_key = fs.product_key
-			WHERE fs.sales_order_number = $$transaction[0];
-			-- AND ...
+		SELECT dp.product_alternate_key, dp.product_name
+		FROM dimension_product dp
+		JOIN fact_sales fs ON
+		dp.product_key = fs.product_key
+		WHERE fs.sales_order_number = $$transaction[0];
+		-- AND ...
 		~;
 		$query = $db->prepare($sql);
 		$query->execute;
 		my@items;
 		while(my($key,$description)=$query->fetchrow){
-			$apriori->{itemsKeyDescription}{$key}=$description;
+			$apriori->{keyItemsDescription}{$key}=$description;
 			push@items,$key;
 		}
 		$apriori->insert_key_items_transaction(\@items);
@@ -674,25 +687,45 @@ The type of metrics
 
 =over 4
 
-=item * minSupport - the minimum support, default value is 0.01(1%);
+=item minSupport
 
-=item * minConfidence - the minimum confidence, default value is 0.10(10%);
+The minimum support(required), default value is 0.01(1%)
 
-=item * minLift - the minimum lift(optional);
+=item minConfidence
 
-=item * minLeverage - the minimum leverage(optional);
+The minimum confidence(required), default value is 0.10(10%)
 
-=item * minConviction - the minimum conviction(optional);
+=item minLift
 
-=item * minCoverage - the minimum coverage(optional);
+The minimum lift(optional)
 
-=item * minCorrelation - the minimum correlation(optional);
+=item minLeverage
 
-=item * minCosine - the minimum cosine(optional);
+The minimum leverage(optional)
 
-=item * minLaplace - the minimum laplace(optional);
+=item minConviction
 
-=item * minJaccard - the minimum jaccard(optional).
+The minimum conviction(optional)
+
+=item minCoverage
+
+The minimum coverage(optional)
+
+=item minCorrelation
+
+The minimum correlation(optional)
+
+=item minCosine
+
+The minimum cosine(optional)
+
+=item minLaplace
+
+The minimum laplace(optional)
+
+=item minJaccard
+
+The minimum jaccard(optional)
 
 =back
 
@@ -702,13 +735,17 @@ The limit of rules(optional)
 
 =head2 output
 
-The output type:(optional)
+The output type(optional):
 
 =over 4
 
-=item * 1 - Text file delimited by tab;
+=item *
 
-=item * 2 - Excel file with chart.
+1 - Text file delimited by tab;
+
+=item *
+
+2 - Excel file with chart.
 
 =back
 
@@ -716,13 +753,13 @@ The output type:(optional)
 
 A value boolean to display the messages(optional)
 
-=head2 itemsKeyDescription
+=head2 keyItemsDescription
 
-Hash table to add items by key and description.
+Hash table reference to add item by key and description.
 
-=head2 itemsKeyTransactions
+=head2 keyItemsTransactions
 
-Hash table, to add the key items per transaction.
+Hash table reference to add items by keys and transactions.
 
 =head2 frequentItemset
 
@@ -732,26 +769,25 @@ Frequent itemset.
 
 A data structure to store the name of the rule, key items, implication, support, confidence, lift, leverage, conviction, coverage, correlation, cosine, laplace and jaccard.
 
-    $self->{associationRules} = {
-                                '1' => {
-                                       'implication' => '{ 102 } => { 101 }',
-                                       'support' => '0.7000',
-                                       'correlation' => '0,5092',
-                                       'jaccard' => '0,7778',
-                                       'confidence' => '0.7778',
-                                       'conviction' => '1,3500',
-                                       'leverage' => '0,0700',
-                                       'laplace' => '0,5862',
-                                       'cosine' => '0,8819',
-                                       'coverage' => '0,9000',
-                                       'lift' => '1,1111',
-                                       'items' => [
-                                                  '102',
-                                                  '101'
-                                                  ]
-                                        }
-                                },
-                                # to be continued...
+	$self->{associationRules} = {
+	                            '1' => {
+	                                   'implication' => '{ 101 } => { 103 }',
+	                                   'jaccard' => '0,8750',
+	                                   'conviction' => '2,4000',
+	                                   'cosine' => '0,9354',
+	                                   'coverage' => '0,8000',
+	                                   'correlation' => '0,7638',
+	                                   'lift' => '1,2500',
+	                                   'leverage' => '0,1400',
+	                                   'laplace' => '0,6071',
+	                                   'support' => '0,7000',
+	                                   'confidence' => '0,8750',
+	                                   'items' => [
+	                                                '101',
+	                                                '103'
+	                                              ]
+	                                 },
+	                            # to be continued...
 
 =head1 METHODS
 
@@ -765,7 +801,9 @@ Insert key items per transaction. Accepts the following arguments:
 
 =over 4
 
-=item * An array reference to key items.
+=item *
+
+An array reference to key items.
 
 =back
 
@@ -775,9 +813,13 @@ Insert items per line(transaction). Accepts the following arguments:
 
 =over 4
 
-=item * Data file;
+=item *
 
-=item * Item separator.
+Data file;
+
+=item *
+
+Item separator.
 
 =back
 
